#include "ds1631.h"

#define WRITE    0
#define READ     1

void ds1631_write_mode() {
	clear_bit(tris_array[ds1631_sda_port - PORTA], ds1631_sda_pin);   // output
}
						 
void ds1631_read_mode() {
	set_bit(tris_array[ds1631_sda_port - PORTA], ds1631_sda_pin);   // output
}

 
void ds1631_start() {

	ds1631_write_mode();
	
	set_pin(ds1631_sda_port, ds1631_sda_pin);
	ds1631_bit_delay();

	set_pin(ds1631_scl_port, ds1631_scl_pin);
	ds1631_bit_delay();
	clear_pin(ds1631_sda_port, ds1631_sda_pin); // sda goes low
	ds1631_bit_delay();
	clear_pin(ds1631_scl_port, ds1631_scl_pin);
}

void ds1631_stop() {
	ds1631_write_mode();
	
	set_pin(ds1631_scl_port, ds1631_scl_pin);
	ds1631_bit_delay();
	clear_pin(ds1631_sda_port, ds1631_sda_pin); // sda goes low
	ds1631_bit_delay();
	set_pin(ds1631_sda_port, ds1631_sda_pin); // then high = stop
}


// sda changes state while scl is low

void ds1631_write_byte(uns8 data) {
uns8 count;

	ds1631_write_mode();
	
	for (count = 0; count < 8; count++) {
		change_pin(ds1631_sda_port, ds1631_sda_pin, data.7);
		ds1631_bit_delay();
		set_pin(ds1631_scl_port, ds1631_scl_pin);
		ds1631_bit_delay();
		clear_pin(ds1631_scl_port, ds1631_scl_pin);
		data = data << 1;
	}	
}
uns8 ds1631_read_and_ack(uns8 last) {

uns8 count, data;
	ds1631_read_mode();
	for (count = 0; count < 8; count++) {
		set_pin(ds1631_scl_port, ds1631_scl_pin);
		ds1631_bit_delay();
		data = data << 1;
		data.0 = test_pin(ds1631_sda_port, ds1631_sda_pin);
		clear_pin(ds1631_scl_port, ds1631_scl_pin);
		ds1631_bit_delay();
	}
	
	ds1631_write_mode();	

	change_pin(ds1631_sda_port, ds1631_sda_pin, last);
	
	set_pin(ds1631_scl_port, ds1631_scl_pin);
	ds1631_bit_delay();
	clear_pin(ds1631_scl_port, ds1631_scl_pin);
	ds1631_bit_delay();
	return data;
}

void ds1631_write_convert(uns8 control, uns8 command) {

	ds1631_start();

	ds1631_write_and_ack(control); 

	ds1631_write_and_ack(command);	

	ds1631_stop();	
}

void ds1631_write_and_ack(uns8 data) {


	ds1631_write_byte(data);
	 
	ds1631_read_mode();

	set_pin(ds1631_scl_port, ds1631_scl_pin);
	ds1631_bit_delay();
	clear_pin(ds1631_scl_port, ds1631_scl_pin);
	
	ds1631_write_mode();

}	

void ds1631_write_config(uns8 control, uns8 command, uns8 data) {

	ds1631_start();
	control.0 = WRITE;

	ds1631_write_and_ack(control);
	ds1631_write_and_ack(command);
	ds1631_write_and_ack(data);

	ds1631_stop();	
}

uns8 ds1631_read_config(uns8 control, uns8 command) {
	
uns8 result;

	ds1631_start();
	control.0 = WRITE; // Set read/^write bit to write
	ds1631_write_and_ack(control);
	ds1631_write_and_ack(command);
	control.0 = READ; // now set to read
	ds1631_start();
	ds1631_write_and_ack(control);

	result = ds1631_read_and_ack(1);
	ds1631_stop();
	return result;
}


uns16 ds1631_get_temp(uns8 control, uns8 command ) {

uns16 result;

	ds1631_start();
	control.0 = WRITE; // Set read/^write bit to write
	ds1631_write_and_ack(control);
	ds1631_write_and_ack(command);
	
	control.0 = READ; // now set to read
	ds1631_start();
	ds1631_write_and_ack(control);
	
	result = ds1631_read_and_ack(0);
	
	result = result << 8;
	result = result + ds1631_read_and_ack(1);
	ds1631_stop();
	return result;
}

void ds1631_setup() {
		make_output(ds1631_sda_port, ds1631_sda_pin);   // output
		make_output(ds1631_scl_port, ds1631_scl_pin);   // output
		set_pin(ds1631_sda_port, ds1631_sda_pin);	// in idle both high
		set_pin(ds1631_scl_port, ds1631_scl_pin);
}

/*
0 =  0000  00 0 00
1 =  0625  06 1 00 
2 =  1250  13 1 00
3 =  1875  19 2 25
4 =  2500  25 3 25
5 =  3125  31 3 25
6 =  3750  38 4 50
7 =  4375  44 4 50
8 =  5000  50 5 50
9 =  5625  56 6 50
10 = 6250  63 6 50
11 = 6875  69 7 75
12 = 7500  75 8 75
13 = 8125  81 8 75
14 = 8750  88 9 up
15 = 9375  94 9 up
*/

uns8 convert_to_dec2b(uns8 data) {
	
	switch (data) {
		case 0: return 00;
		case 1: return 00;
		case 2: return 00;
		case 3: return 25;
		case 4: return 25;
		case 5: return 25;
		case 6: return 50;
		case 7: return 50;
		case 8: return 50;
		case 9: return 50;
		case 10: return 50;
		case 11: return 75;
		case 12: return 75;
		case 13: return 75;
		case 14: return 99;
		case 15: return 99;
	}	
}

uns8 convert_to_dec1(uns8 data) {
	
	switch (data) {
		case 0: return 0;
		case 1: return 1;
		case 2: return 1;
		case 3: return 2;
		case 4: return 3;
		case 5: return 3;
		case 6: return 4;
		case 7: return 4;
		case 8: return 5;
		case 9: return 6;
		case 10: return 6;
		case 11: return 7;
		case 12: return 8;
		case 13: return 8;
		case 14: return 9;
		case 15: return 9;
	}	
}
uns8 convert_to_dec2(uns8 data) {
	
	switch (data) {
		case 0: return 00;
		case 1: return 06;
		case 2: return 13;
		case 3: return 19;
		case 4: return 25;
		case 5: return 31;
		case 6: return 38;
		case 7: return 44;
		case 8: return 50;
		case 9: return 56;
		case 10: return 63;
		case 11: return 69;
		case 12: return 75;
		case 13: return 81;
		case 14: return 88;
		case 15: return 94;
	}	
}

