#ifndef __pic_rf_h
#define __pic_rf_h defined

#include "config.h"
#include "pic_utils.h"

// Copy this into your config.h

/*  

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// pic_rf_24l01 include
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define rf_ce_port PORTB
#define rf_ce_pin  2

#define rf_csn_port PORTB
#define rf_csn_pin  1

#define rf_irq_port PORTB
#define rf_irq_pin  0

#define rf_mosi_port PORTB
#define rf_mosi_pin 3	

#define rf_miso_port PORTB
#define rf_miso_pin 4

#define rf_irq_port PORTB
#define rf_irq_pin 5

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

*/



struct _rf_config {
	uns8 payload_width_ch2;
	uns8 payload_width_ch1;
	uns8 address_ch2[5];
	uns8 address_ch1[5];
	uns8 address_width; // (6 bits)
	uns8 crystal; 	// (3 bits)
	uns8 output_power; 	// (2 bits)
	uns8 channel; 	// (7 bits);
	uns8 options;
};	

typedef struct _rf_config rf_config; 

// rf_config options bits

#define ENABLE_RECEIVE		0
#define LONG_CRC			1
#define ENABLE_CRC			2

#define ENABLE_1_MBPS		5
#define ENABLE_SHOCKBURST	6
#define ENABLE_CH2			7

#define RECEIVE_MODE 1
#define TRANSMIT_MODE 0

// read / write registers
#define RF_WR_REG_CONFIG_REG	0b00100000
#define RF_RD_REG_CONFIG_REG    0b00000000
#define RF_WR_REG_SETUP_RETR	0b00100100
#define RF_WR_REG_SETUP_AW 		0b00100011
#define RF_WR_REG_RF_SETUP 		0b00100110
#define RF_WR_REG_TX_ADDR 		0b00110000
#define RF_WR_REG_RX_ADDR_P0 	0b00101010
#define RF_WR_REG_EN_AA 		0b00100001
#define RF_WR_REG_RX_PW_P0 		0b00110001
#define RF_WR_REG_RF_CH         0b00100101
#define RF_WR_REG_STATUS        0b00100111
#define RF_RD_REG_STATUS        0b00000111
#define RF_RD_REG_FIFO_STATUS   0b00010111

#define RF_FLUSH_TX				0b11100001
#define RF_FLUSH_RX				0b11100010

// commands
#define RF_W_TX_PAYLOAD		0b10100000
#define RF_R_RX_PAYLOAD 	0b01100001

#define RF_NOP				0b11111111
// bits
#define CONF_PRIM_RX	0
#define CONF_PWR_UP		1
#define CONF_CRC0		2
#define CONF_EN_CRC		3

#define STAT_RX_DR		6
#define STAT_TX_DS		5
#define MAX_RT			4
#define TX_FULL			0


static bit rf_current_mode_receive = 0;
static uns8 rf_current_channel = 2;

void pic_rf_quick_init(char *my_config, uns8 my_channel, bit my_receive_on); //-- used

void pic_rf_setup(); //-- used done
void pic_rf_init(rf_config *my_config);
void pic_rf_transmit(uns8 *data, uns8 bytes_to_transmit); //-- used
void pic_rf_set_channel(uns8 channel);  //-- should be used
uns8 pic_rf_handle_irq_receive(uns8 *data, uns8 bytes_to_receive);

// Internal routines
uns8 pic_rf_read_register(uns8 cmd, uns8 *data, uns8 data_len); // returns status
void pic_rf_send_bytes(char *bytes, uns8 num_bytes);
uns8 pic_rf_send_byte(uns8 b); 	// returns status
uns8 pic_rf_send_byte_int(uns8 b); // second version for interrupts
//uns8 inline pic_rf_read_register_int(uns8 cmd, uns8 *data, uns8 data_len); // returns status
void pic_rf_set_mode(uns8 mode);
uns8 pic_rf_send_command(uns8 cmd, uns8 *data, uns8 data_len);

//#define pic_rf_read_register(cmd, data, data_len) \
//	pic_rf_send_command(cmd, data, data_len)

inline uns8 pic_rf_read_register_inline(uns8 cmd, uns8 *data, uns8 data_len) {

uns8 byte_counter, status;

    clear_pin(rf_csn_port, rf_csn_pin);

	status = pic_rf_send_byte_int(cmd);
	 
	for(byte_counter = 0 ; byte_counter < data_len ; byte_counter++) {
		data[byte_counter] = pic_rf_send_byte_int(0); // dummy send to get byte back
	}
	
    set_pin(rf_csn_port, rf_csn_pin);
    
    return status;
}	

inline uns8 pic_rf_send_command_inline(uns8 cmd, uns8 *data, uns8 data_len) {

uns8 byte_counter, status;

    clear_pin(rf_csn_port, rf_csn_pin);//    print_str("cmd:");
    //print_int_hex(cmd);
    //print_str(" ");
	status = pic_rf_send_byte_int(cmd); 
	for(byte_counter = 0 ; byte_counter < data_len ; byte_counter++) {
		//print_str("d: ");
		//print_int_hex(data[byte_counter]);
		//print_str(" ");
		pic_rf_send_byte_int(data[byte_counter]);
	}							 
	
    set_pin(rf_csn_port, rf_csn_pin);
		//print_str("st: ");
		//print_int_hex(status);
		//print_str(" ");
    //delay_ms(200);
    return status;
}	


inline void pic_rf_receive_inline(uns8 *data, uns8 bytes_to_receive) {
        pic_rf_read_register_inline(RF_R_RX_PAYLOAD, data, bytes_to_receive);
}

#define pic_rf_get_status() \
	    pic_rf_read_register(RF_NOP, 0, 0)

#define pic_rf_set_status(status) \
	pic_rf_send_command(RF_WR_REG_STATUS, status, 1)

#define pic_rf_receive_mode() pic_rf_set_mode(RECEIVE_MODE)
#define pic_rf_transmit_mode() pic_rf_set_mode(TRANSMIT_MODE)



// #define catches

#ifndef rf_ce_port
	#error "pic_rf: You haven't defined rf_ce_port in your config.h!"
#endif
#ifndef rf_ce_pin
	#error "pic_rf: You haven't defined rf_ce_pin in your config.h!"
#endif

#ifndef rf_csn_port
	#error "pic_rf: You haven't defined rf_csn_port in your config.h!"
#endif
#ifndef rf_csn_pin
	#error "pic_rf: You haven't defined rf_csn_pin in your config.h!"
#endif

#ifndef rf_irq_port
	#error "pic_rf: You haven't defined rf_irq_port in your config.h!"
#endif
#ifndef rf_irq_pin
	#error "pic_rf: You haven't defined rf_irq_pin in your config.h!"
#endif

#ifndef rf_mosi_port
	#error "pic_rf: You haven't defined rf_mosi_port in your config.h!"
#endif
#ifndef rf_mosi_pin
	#error "pic_rf: You haven't defined rf_mosi_pin in your config.h!"
#endif

#ifndef rf_miso_port
	#error "pic_rf: You haven't defined rf_miso_port in your config.h!"
#endif
#ifndef rf_miso_pin
	#error "pic_rf: You haven't defined rf_miso_pin in your config.h!"
#endif

#ifndef rf_sck_port
	#error "pic_rf: You haven't defined rf_sck_port in your config.h!"
#endif
#ifndef rf_sck_pin
	#error "pic_rf: You haven't defined rf_sck_pin in your config.h!"
#endif

#endif


// notes

//clear CE to activate rx monitoring of packets
//pulse CE to transmit data
//standby to tx/rx = 130us